/*****************************************************************************
    util.h

    Diese Datei ist Teil des Programmpakets 'ACL-tftp'.
    Alle Rechte bei Andreas G. Schindler
                    Klappacher Str. 2i
                    64285 Darmstadt, Deutschland.

    Dieses Programm ist 'Freie Software'; Sie knnen es entsprechend den 
    Bedingungen der 'Allgemeinen ffentlichen GNU-Lizenz' (verffentlicht 
    durch die 'Free Software Foundation') weiter verbreiten und/oder 
    modifizieren. Massgeblich ist die Version 2 oder eine neuere Version
    dieser Lizenz. 

    Dieses Programm wird in der Hoffnung dass es ntzlich is weitergegeben,
    jedoch OHNE JEDE GARANTIE. Dieser Garantieauschluss bezieht sich auch
    auf Eigenschaften wie Marktfhigkeit oder Eignung zu einem bestimmten
    Zweck. Details finden sich in der 'Allgemeinen ffentlichen GNU-Lizenz'.

    Eine Kopie dieser Lizenz sollte in  Programmpaket, dem diese Datei
    entstammt, enthalten sein. Falls nicht, schreiben Sie an die 
    'Free Software Foundation, Inc.', 59 Temple Place, Suite 330, Boston, 
    MA  02111-1307  USA.


    This file is part of the 'ACL-tftp' Package
    Copyright (C) 1999-2005 by Andreas G. Schindler
                               Klappacher Str. 2i
                               Darmstadt, Germany.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*****************************************************************************/
#ifndef UTIL_H
#define UTIL_H

#include <netinet/in.h>
#include <sys/stat.h>
#include <sys/times.h>
#include <pwd.h>
#include <grp.h>
#include "types.h"
#include "realpath.h"

#define T2A_XFER	1
#define T2A_SETUP	2
#define T2A_RET		4
#define T2A_SPEC	8
#define T2A_ALL		(T2A_XFER|T2A_SETUP|T2A_RET|T2A_SPEC)

#define ACC_READ	1
#define ACC_WRITE	2
#define ACC_CREATE	3
#define ACC_EXEC	8

#define EBS_SEND	1
#define EBS_SACK	2
#define EBS_RECV	3
#define EBS_RACK	4
#define EBS_CONN	5
#define EBS_IDLE	6

#include "patstr.h"

#ifndef strndup
char *strndup(const char *src, size_t maxlen);
#endif /* strndup */
#ifndef strncpy
char *strncpy(char *dst, const char *src, size_t maxlen);
#endif /* strncpy */
#ifndef stpcpy
char *stpcpy(char *dst, const char *src); 
#endif /* stpcpy */
#ifndef stpncpy
char *stpncpy(char *dst, const char *src, size_t maxlen);
#endif /* stpncpy */
void version(FILE *fp, char *me);
void logBuild(void);
void showBuild(FILE *fp);
void copyright(FILE *fp, int short_form);
int getPathMax(void);
char *baseName (char *path, int dyn);
char *dirName (char *path, int dyn);
int isGroupMember (gid_t gid);
int fileAccess (char *file, int check);
int dirAccess(char *dir, int check);
int dpstat (char *filename, struct stat *sb);
void trim (char *txt);
char *xstrtok (char *buf, char *pat, char **context);
char *qstrtok (char *buf, char *pat, char **context, char *quotes);
char *zstrtok (char *buf, char *pat, char **context);
char *zstrtok0 (char *buf, size_t buflen, char **context, size_t *remlen, size_t *reslen);
int checkFileMap (char *map);
char *xlateFileMap (char *path, char *map, struct sockaddr_in *client, char *realpath, int dir);
char *xlatePath (char *path);
char *fmtstat (struct stat *st);
int iscidr (in_addr_t mask);
char *cidrsplit(char *cidr);
in_addr_t cidr2mask(int cidrbits);
char *ccidr2mask(int cidrbits);
int i2ar(int i, char *a, size_t alen);
char *i2a(int i);
int stricmp (char *s1, char *s2);
int strnicmp (char *s1, char *s2, size_t len);
int cutLine (char *line,int *argc,char **argv,int maxargs,char **ctxs);
void lbitSet (ulong *mask, int bitNo);
void lbitClr (ulong *mask, int bitNo);
int lbitTst (ulong *mask, int bitNo);
int lbitTstClr (ulong *mask, int bitNo);
int lbitTstSet (ulong *mask, int bitNo);
char *dynMemStat(void);
struct timeval uftime(struct timeval *from);
int initBackoff(tout_t *top, double strength);
struct timeval proBackoff(tout_t *top, int selector, int curRetry);
char * listBackoff(tout_t *top, int selector);
int asc2tv(char *numstr, struct timeval *tvp);
double tv2double(struct timeval *tvp);
struct timeval double2tv(double d);
struct timeval int2tv(long ip, long fp);
char *tv2asc (struct timeval *tvp);
char *timer2asc (tout_t *tp, int what);
struct timeval tvDiff(struct timeval t1, struct timeval t2);
struct timeval tvSum(struct timeval t1, struct timeval t2);
struct timeval tvDivide(struct timeval t1, int fac);
int tvCmpGT(struct timeval t1, struct timeval t2);
int tvCmpGE(struct timeval t1, struct timeval t2);
int parseTimes(char *arg, tout_t *tp);
int parseCTimes(char *arglist, tout_t *tp);
void startTimer(V_L struct timeval *timer);
int checkTimer(struct timeval timer, struct timeval compare_to);
char *strdupNoNull(char *string, int len, char repl);
char *mflag2name(int flag);
char *mflag2txt (int flag);
char shortMF (int flag);
int globCheck(const char *func, int line, int *Mflag, char *name, int flag);
int txt2uid(char *text, uid_t *uid);
int txt2gid(char *text, gid_t *gid);
int cvtunit(char *text, ulong *unit);
int parseCmr(char *s, dlb_t **bl);
void initLocale(char *locale);
int patstr(patlist_t *pl, char *arg, char *sep, int flag);
char *bmap2pat(int val, patlist_t *pl, char *sep);
int powerOf2 (int val);
#endif /* UTIL_H */

